@@ -9,7 +9,7 @@ module Enumerable
   #
   # Example:
   #
-  #   latest_transcripts.group_by(&:day).each do |day, transcripts| 
+  #   latest_transcripts.group_by(&:day).each do |day, transcripts|
   #     p "#{day} -> #{transcripts.map(&:class).join(', ')}"
   #   end
   #   "2006-03-01 -> Transcript"
@@ -42,7 +42,7 @@ module Enumerable
   #
   # The latter is a shortcut for:
   #
-  #  payments.inject { |sum, p| sum + p.price }
+  #  payments.inject(0) { |sum, p| sum + p.price }
   #
   # It can also calculate the sum without the use of a block.
   #
@@ -55,12 +55,10 @@ module Enumerable
   #  [].sum(Payment.new(0)) { |i| i.amount } # => Payment.new(0)
   #
   def sum(identity = 0, &block)
-    return identity unless size > 0
-
     if block_given?
-      map(&block).sum
+      map(&block).sum(identity)
     else
-      inject { |sum, element| sum + element }
+      inject { |sum, element| sum + element } || identity
     end
   end
 
@@ -68,7 +66,8 @@ module Enumerable
   # +memo+ to the block. Handy for building up hashes or
   # reducing collections down to one object. Examples:
   #
-  #   %w(foo bar).each_with_object({}) { |str, hsh| hsh[str] = str.upcase } #=> {'foo' => 'FOO', 'bar' => 'BAR'}
+  #   %w(foo bar).each_with_object({}) { |str, hsh| hsh[str] = str.upcase }
+  #   # => {'foo' => 'FOO', 'bar' => 'BAR'}
   #
   # *Note* that you can't use immutable objects like numbers, true or false as
   # the memo. You would think the following returns 120, but since the memo is
@@ -77,11 +76,10 @@ module Enumerable
   #   (1..5).each_with_object(1) { |value, memo| memo *= value } # => 1
   #
   def each_with_object(memo, &block)
-    returning memo do |m|
-      each do |element|
-        block.call(element, m)
-      end
+    each do |element|
+      block.call(element, memo)
     end
+    memo
   end unless [].respond_to?(:each_with_object)
 
   # Convert an enumerable to a hash. Examples:
@@ -90,18 +88,33 @@ module Enumerable
   #     => { "nextangle" => <Person ...>, "chade-" => <Person ...>, ...}
   #   people.index_by { |person| "#{person.first_name} #{person.last_name}" }
   #     => { "Chade- Fowlersburg-e" => <Person ...>, "David Heinemeier Hansson" => <Person ...>, ...}
-  # 
+  #
   def index_by
     inject({}) do |accum, elem|
       accum[yield(elem)] = elem
       accum
     end
   end
-  
+
   # Returns true if the collection has more than 1 element. Functionally equivalent to collection.size > 1.
   # Works with a block too ala any?, so people.many? { |p| p.age > 26 } # => returns true if more than 1 person is over 26.
   def many?(&block)
     size = block_given? ? select(&block).size : self.size
     size > 1
   end
+
+  # The negative of the Enumerable#include?. Returns true if the collection does not include the object.
+  def exclude?(object)
+    !include?(object)
+  end
+end
+
+class Range #:nodoc:
+  # Optimize range sum to use arithmetic progression if a block is not given and
+  # we have a range of numeric values.
+  def sum(identity = 0)
+    return super if block_given? || !(first.instance_of?(Integer) && last.instance_of?(Integer))
+    actual_last = exclude_end? ? (last - 1) : last
+    (actual_last - first + 1) * (actual_last + first) / 2
+  end
 end
