@@ -1,3 +1,16 @@
+require 'set'
+require 'thread'
+require 'pathname'
+require 'active_support/core_ext/module/aliasing'
+require 'active_support/core_ext/module/attribute_accessors'
+require 'active_support/core_ext/module/introspection'
+require 'active_support/core_ext/module/anonymous'
+require 'active_support/core_ext/object/blank'
+require 'active_support/core_ext/load_error'
+require 'active_support/core_ext/name_error'
+require 'active_support/core_ext/string/starts_ends_with'
+require 'active_support/inflector'
+
 module ActiveSupport #:nodoc:
   module Dependencies #:nodoc:
     extend self
@@ -16,24 +29,27 @@ module ActiveSupport #:nodoc:
 
     # Should we load files or require them?
     mattr_accessor :mechanism
-    self.mechanism = :load
+    self.mechanism = ENV['NO_RELOAD'] ? :require : :load
 
     # The set of directories from which we may automatically load files. Files
     # under these directories will be reloaded on each request in development mode,
-    # unless the directory also appears in load_once_paths.
-    mattr_accessor :load_paths
-    self.load_paths = []
+    # unless the directory also appears in autoload_once_paths.
+    mattr_accessor :autoload_paths
+    self.autoload_paths = []
 
     # The set of directories from which automatically loaded constants are loaded
-    # only once. All directories in this set must also be present in +load_paths+.
-    mattr_accessor :load_once_paths
-    self.load_once_paths = []
+    # only once. All directories in this set must also be present in +autoload_paths+.
+    mattr_accessor :autoload_once_paths
+    self.autoload_once_paths = []
 
     # An array of qualified constant names that have been loaded. Adding a name to
     # this array will cause it to be unloaded the next time Dependencies are cleared.
     mattr_accessor :autoloaded_constants
     self.autoloaded_constants = []
 
+    mattr_accessor :references
+    self.references = {}
+
     # An array of constant names that need to be unloaded on every request. Used
     # to allow arbitrary constants to be marked for unloading.
     mattr_accessor :explicitly_unloadable_constants
@@ -47,34 +63,134 @@ module ActiveSupport #:nodoc:
     mattr_accessor :log_activity
     self.log_activity = false
 
+    # The WatchStack keeps a stack of the modules being watched as files are loaded.
+    # If a file in the process of being loaded (parent.rb) triggers the load of 
+    # another file (child.rb) the stack will ensure that child.rb handles the new 
+    # constants.
+    #
+    # If child.rb is being autoloaded, its constants will be added to
+    # autoloaded_constants. If it was being `require`d, they will be discarded.
+    #
+    # This is handled by walking back up the watch stack and adding the constants
+    # found by child.rb to the list of original constants in parent.rb
+    class WatchStack < Hash
+      # @watching is a stack of lists of constants being watched. For instance,
+      # if parent.rb is autoloaded, the stack will look like [[Object]]. If parent.rb
+      # then requires namespace/child.rb, the stack will look like [[Object], [Namespace]].
+
+      def initialize
+        @watching = []
+        super { |h,k| h[k] = [] }
+      end
+
+      # return a list of new constants found since the last call to watch_modules
+      def new_constants
+        constants = []
+
+        # Grab the list of namespaces that we're looking for new constants under
+        @watching.last.each do |namespace|
+          # Retrieve the constants that were present under the namespace when watch_modules
+          # was originally called
+          original_constants = self[namespace].last
+
+          mod = Inflector.constantize(namespace) if Dependencies.qualified_const_defined?(namespace)
+          next unless mod.is_a?(Module)
+
+          # Get a list of the constants that were added
+          new_constants = mod.local_constant_names - original_constants
+
+          # self[namespace] returns an Array of the constants that are being evaluated
+          # for that namespace. For instance, if parent.rb requires child.rb, the first
+          # element of self[Object] will be an Array of the constants that were present
+          # before parent.rb was required. The second element will be an Array of the
+          # constants that were present before child.rb was required.
+          self[namespace].each do |namespace_constants|
+            namespace_constants.concat(new_constants)
+          end
+
+          # Normalize the list of new constants, and add them to the list we will return
+          new_constants.each do |suffix|
+            constants << ([namespace, suffix] - ["Object"]).join("::")
+          end
+        end
+        constants
+      ensure
+        # A call to new_constants is always called after a call to watch_modules
+        pop_modules(@watching.pop)
+      end
+
+      # Add a set of modules to the watch stack, remembering the initial constants
+      def watch_namespaces(namespaces)
+        watching = []
+        namespaces.map do |namespace|
+          module_name = Dependencies.to_constant_name(namespace)
+          original_constants = Dependencies.qualified_const_defined?(module_name) ?
+            Inflector.constantize(module_name).local_constant_names : []
+
+          watching << module_name
+          self[module_name] << original_constants
+        end
+        @watching << watching
+      end
+
+      def pop_modules(modules)
+        modules.each { |mod| self[mod].pop }
+      end
+    end
+
     # An internal stack used to record which constants are loaded by any block.
     mattr_accessor :constant_watch_stack
-    self.constant_watch_stack = []
+    self.constant_watch_stack = WatchStack.new
 
     # Module includes this module
     module ModuleConstMissing #:nodoc:
-      def self.included(base) #:nodoc:
+      def self.append_features(base)
         base.class_eval do
-          unless defined? const_missing_without_dependencies
-            alias_method_chain :const_missing, :dependencies
-          end
+          # Emulate #exclude via an ivar
+          return if defined?(@_const_missing) && @_const_missing
+          @_const_missing = instance_method(:const_missing)
+          remove_method(:const_missing)
         end
+        super
       end
 
-      def self.excluded(base) #:nodoc:
+      def self.exclude_from(base)
         base.class_eval do
-          if defined? const_missing_without_dependencies
-            undef_method :const_missing
-            alias_method :const_missing, :const_missing_without_dependencies
-            undef_method :const_missing_without_dependencies
-          end
+          define_method :const_missing, @_const_missing
+          @_const_missing = nil
         end
       end
 
       # Use const_missing to autoload associations so we don't have to
       # require_association when using single-table inheritance.
-      def const_missing_with_dependencies(class_id)
-        ActiveSupport::Dependencies.load_missing_constant self, class_id
+      def const_missing(const_name, nesting = nil)
+        klass_name = name.presence || "Object"
+
+        if !nesting
+          # We'll assume that the nesting of Foo::Bar is ["Foo::Bar", "Foo"]
+          # even though it might not be, such as in the case of
+          # class Foo::Bar; Baz; end
+          nesting = []
+          klass_name.to_s.scan(/::|$/) { nesting.unshift $` }
+        end
+
+        # If there are multiple levels of nesting to search under, the top
+        # level is the one we want to report as the lookup fail.
+        error = nil
+
+        nesting.each do |namespace|
+          begin
+            return Dependencies.load_missing_constant Inflector.constantize(namespace), const_name
+          rescue NoMethodError then raise
+          rescue NameError => e
+            error ||= e
+          end
+        end
+
+        # Raise the first error for this set. If this const_missing came from an
+        # earlier const_missing, this will result in the real error bubbling
+        # all the way up
+        raise error
       end
 
       def unloadable(const_desc = self)
@@ -82,81 +198,45 @@ module ActiveSupport #:nodoc:
       end
     end
 
-    # Class includes this module
-    module ClassConstMissing #:nodoc:
-      def const_missing(const_name)
-        if [Object, Kernel].include?(self) || parent == self
-          super
-        else
-          begin
-            begin
-              Dependencies.load_missing_constant self, const_name
-            rescue NameError
-              parent.send :const_missing, const_name
-            end
-          rescue NameError => e
-            # Make sure that the name we are missing is the one that caused the error
-            parent_qualified_name = Dependencies.qualified_name_for parent, const_name
-            raise unless e.missing_name? parent_qualified_name
-            qualified_name = Dependencies.qualified_name_for self, const_name
-            raise NameError.new("uninitialized constant #{qualified_name}").copy_blame!(e)
-          end
-        end
-      end
-    end
-
     # Object includes this module
     module Loadable #:nodoc:
-      def self.included(base) #:nodoc:
-        base.class_eval do
-          unless defined? load_without_new_constant_marking
-            alias_method_chain :load, :new_constant_marking
-          end
-        end
-      end
-
-      def self.excluded(base) #:nodoc:
-        base.class_eval do
-          if defined? load_without_new_constant_marking
-            undef_method :load
-            alias_method :load, :load_without_new_constant_marking
-            undef_method :load_without_new_constant_marking
-          end
-        end
+      def self.exclude_from(base)
+        base.class_eval { define_method(:load, Kernel.instance_method(:load)) }
       end
 
       def require_or_load(file_name)
         Dependencies.require_or_load(file_name)
       end
 
-      def require_dependency(file_name)
-        Dependencies.depend_on(file_name)
+      def require_dependency(file_name, message = "No such file to load -- %s")
+        unless file_name.is_a?(String)
+          raise ArgumentError, "the file name must be a String -- you passed #{file_name.inspect}"
+        end
+
+        Dependencies.depend_on(file_name, false, message)
       end
 
       def require_association(file_name)
         Dependencies.associate_with(file_name)
       end
 
-      def load_with_new_constant_marking(file, *extras) #:nodoc:
+      def load_dependency(file)
         if Dependencies.load?
-          Dependencies.new_constants_in(Object) { load_without_new_constant_marking(file, *extras) }
+          Dependencies.new_constants_in(Object) { yield }.presence
         else
-          load_without_new_constant_marking(file, *extras)
+          yield
         end
       rescue Exception => exception  # errors from loading file
         exception.blame_file! file
         raise
       end
 
-      def require(file, *extras) #:nodoc:
-        if Dependencies.load?
-          Dependencies.new_constants_in(Object) { super }
-        else
-          super
-        end
-      rescue Exception => exception  # errors from required file
-        exception.blame_file! file
-        raise
+      def load(file, *)
+        load_dependency(file) { super }
+      end
+
+      def require(file, *)
+        load_dependency(file) { super }
       end
 
       # Mark the given constant as unloadable. Unloadable constants are removed each
@@ -199,16 +279,15 @@ module ActiveSupport #:nodoc:
     end
 
     def hook!
-      Object.instance_eval { include Loadable }
-      Module.instance_eval { include ModuleConstMissing }
-      Class.instance_eval { include ClassConstMissing }
-      Exception.instance_eval { include Blamable }
+      Object.class_eval { include Loadable }
+      Module.class_eval { include ModuleConstMissing }
+      Exception.class_eval { include Blamable }
       true
     end
 
     def unhook!
-      ModuleConstMissing.excluded(Module)
-      Loadable.excluded(Object)
+      ModuleConstMissing.exclude_from(Module)
+      Loadable.exclude_from(Object)
       true
     end
 
@@ -216,11 +295,16 @@ module ActiveSupport #:nodoc:
       mechanism == :load
     end
 
-    def depend_on(file_name, swallow_load_errors = false)
+    def depend_on(file_name, swallow_load_errors = false, message = "No such file to load -- %s.rb")
       path = search_for_file(file_name)
       require_or_load(path || file_name)
-    rescue LoadError
-      raise unless swallow_load_errors
+    rescue LoadError => load_error
+      unless swallow_load_errors
+        if file_name = load_error.message[/ -- (.*?)(\.rb)?$/, 1]
+          raise LoadError.new(message % file_name).copy_blame!(load_error)
+        end
+        raise
+      end
     end
 
     def associate_with(file_name)
@@ -273,65 +357,53 @@ module ActiveSupport #:nodoc:
 
     # Is the provided constant path defined?
     def qualified_const_defined?(path)
-      raise NameError, "#{path.inspect} is not a valid constant name!" unless
-        /^(::)?([A-Z]\w*)(::[A-Z]\w*)*$/ =~ path
+      names = path.sub(/^::/, '').to_s.split('::')
 
-      names = path.to_s.split('::')
-      names.shift if names.first.empty?
-
-      # We can't use defined? because it will invoke const_missing for the parent
-      # of the name we are checking.
       names.inject(Object) do |mod, name|
-        return false unless uninherited_const_defined?(mod, name)
+        return false unless local_const_defined?(mod, name)
         mod.const_get name
       end
-      return true
     end
 
     if Module.method(:const_defined?).arity == 1
       # Does this module define this constant?
-      # Wrapper to accomodate changing Module#const_defined? in Ruby 1.9
-      def uninherited_const_defined?(mod, const)
+      # Wrapper to accommodate changing Module#const_defined? in Ruby 1.9
+      def local_const_defined?(mod, const)
         mod.const_defined?(const)
       end
     else
-      def uninherited_const_defined?(mod, const) #:nodoc:
+      def local_const_defined?(mod, const) #:nodoc:
         mod.const_defined?(const, false)
       end
     end
 
     # Given +path+, a filesystem path to a ruby file, return an array of constant
     # paths which would cause Dependencies to attempt to load this file.
-    def loadable_constants_for_path(path, bases = load_paths)
+    def loadable_constants_for_path(path, bases = autoload_paths)
       path = $1 if path =~ /\A(.*)\.rb\Z/
       expanded_path = File.expand_path(path)
+      paths = []
 
-      bases.collect do |root|
+      bases.each do |root|
         expanded_root = File.expand_path(root)
         next unless %r{\A#{Regexp.escape(expanded_root)}(/|\\)} =~ expanded_path
 
         nesting = expanded_path[(expanded_root.size)..-1]
         nesting = nesting[1..-1] if nesting && nesting[0] == ?/
         next if nesting.blank?
-        nesting_camel = nesting.camelize
-        begin
-          qualified_const_defined?(nesting_camel)
-        rescue NameError
-          next
-        end
 
-        [
-          nesting.camelize,
-          # Special case: application.rb might define ApplicationControlller.
-          ('ApplicationController' if nesting == 'application')
-        ]
-      end.flatten.compact.uniq
+        paths << nesting.camelize
+      end
+
+      paths.uniq!
+      paths
     end
 
-    # Search for a file in load_paths matching the provided suffix.
+    # Search for a file in autoload_paths matching the provided suffix.
     def search_for_file(path_suffix)
-      path_suffix = path_suffix + '.rb' unless path_suffix.ends_with? '.rb'
-      load_paths.each do |root|
+      path_suffix = path_suffix.sub(/(\.rb)?$/, ".rb")
+
+      autoload_paths.each do |root|
         path = File.join(root, path_suffix)
         return path if File.file? path
       end
@@ -341,14 +413,14 @@ module ActiveSupport #:nodoc:
     # Does the provided path_suffix correspond to an autoloadable module?
     # Instead of returning a boolean, the autoload base for this module is returned.
     def autoloadable_module?(path_suffix)
-      load_paths.each do |load_path|
+      autoload_paths.each do |load_path|
         return load_path if File.directory? File.join(load_path, path_suffix)
       end
       nil
     end
 
     def load_once_path?(path)
-      load_once_paths.any? { |base| path.starts_with? base }
+      autoload_once_paths.any? { |base| path.starts_with? base }
     end
 
     # Attempt to autoload the provided module name by searching for a directory
@@ -360,7 +432,7 @@ module ActiveSupport #:nodoc:
       return nil unless base_path = autoloadable_module?(path_suffix)
       mod = Module.new
       into.const_set const_name, mod
-      autoloaded_constants << qualified_name unless load_once_paths.include?(base_path)
+      autoloaded_constants << qualified_name unless autoload_once_paths.include?(base_path)
       return mod
     end
 
@@ -379,7 +451,7 @@ module ActiveSupport #:nodoc:
 
       result = nil
       newly_defined_paths = new_constants_in(*parent_paths) do
-        result = load_without_new_constant_marking path
+        result = Kernel.load path
       end
 
       autoloaded_constants.concat newly_defined_paths unless load_once_path?(path)
@@ -391,7 +463,7 @@ module ActiveSupport #:nodoc:
     # Return the constant path for the provided parent and constant name.
     def qualified_name_for(mod, name)
       mod_name = to_constant_name mod
-      (%w(Object Kernel).include? mod_name) ? name.to_s : "#{mod_name}::#{name}"
+      mod_name == "Object" ? name.to_s : "#{mod_name}::#{name}"
     end
 
     # Load the constant named +const_name+ which is missing from +from_mod+. If
@@ -399,38 +471,30 @@ module ActiveSupport #:nodoc:
     # using const_missing.
     def load_missing_constant(from_mod, const_name)
       log_call from_mod, const_name
-      if from_mod == Kernel
-        if ::Object.const_defined?(const_name)
-          log "Returning Object::#{const_name} for Kernel::#{const_name}"
-          return ::Object.const_get(const_name)
-        else
-          log "Substituting Object for Kernel"
-          from_mod = Object
-        end
-      end
 
-      # If we have an anonymous module, all we can do is attempt to load from Object.
-      from_mod = Object if from_mod.name.blank?
-
-      unless qualified_const_defined?(from_mod.name) && from_mod.name.constantize.object_id == from_mod.object_id
+      unless qualified_const_defined?(from_mod.name) && Inflector.constantize(from_mod.name).equal?(from_mod)
         raise ArgumentError, "A copy of #{from_mod} has been removed from the module tree but is still active!"
       end
 
-      raise ArgumentError, "#{from_mod} is not missing constant #{const_name}!" if uninherited_const_defined?(from_mod, const_name)
+      raise ArgumentError, "#{from_mod} is not missing constant #{const_name}!" if local_const_defined?(from_mod, const_name)
 
       qualified_name = qualified_name_for from_mod, const_name
       path_suffix = qualified_name.underscore
+
+      trace = caller.reject {|l| l =~ %r{#{Regexp.escape(__FILE__)}}}
       name_error = NameError.new("uninitialized constant #{qualified_name}")
+      name_error.set_backtrace(trace)
 
       file_path = search_for_file(path_suffix)
+
       if file_path && ! loaded.include?(File.expand_path(file_path)) # We found a matching file to load
         require_or_load file_path
-        raise LoadError, "Expected #{file_path} to define #{qualified_name}" unless uninherited_const_defined?(from_mod, const_name)
+        raise LoadError, "Expected #{file_path} to define #{qualified_name}" unless local_const_defined?(from_mod, const_name)
         return from_mod.const_get(const_name)
       elsif mod = autoload_module!(from_mod, const_name, qualified_name, path_suffix)
         return mod
       elsif (parent = from_mod.parent) && parent != from_mod &&
-            ! from_mod.parents.any? { |p| uninherited_const_defined?(p, const_name) }
+            ! from_mod.parents.any? { |p| local_const_defined?(p, const_name) }
         # If our parents do not have a constant named +const_name+ then we are free
         # to attempt to load upwards. If they do have such a constant, then this
         # const_missing must be due to from_mod::const_name, which should not
@@ -447,17 +511,50 @@ module ActiveSupport #:nodoc:
     end
 
     # Remove the constants that have been autoloaded, and those that have been
-    # marked for unloading.
+    # marked for unloading. Before each constant is removed a callback is sent
+    # to its class/module if it implements +before_remove_const+.
+    #
+    # The callback implementation should be restricted to cleaning up caches, etc.
+    # as the enviroment will be in an inconsistent state, e.g. other constants
+    # may have already been unloaded and not accessible.
     def remove_unloadable_constants!
       autoloaded_constants.each { |const| remove_constant const }
       autoloaded_constants.clear
+      Reference.clear!
       explicitly_unloadable_constants.each { |const| remove_constant const }
     end
 
+    class Reference
+      @@constants = Hash.new { |h, k| h[k] = Inflector.constantize(k) }
+
+      attr_reader :name
+
+      def initialize(name)
+        @name = name.to_s
+        @@constants[@name] = name if name.respond_to?(:name)
+      end
+
+      def get
+        @@constants[@name]
+      end
+
+      def self.clear!
+        @@constants.clear
+      end
+    end
+
+    def ref(name)
+      references[name] ||= Reference.new(name)
+    end
+
+    def constantize(name)
+      ref(name).get
+    end
+
     # Determine if the given constant has been automatically loaded.
     def autoloaded?(desc)
       # No name => anonymous module.
-      return false if desc.is_a?(Module) && desc.name.blank?
+      return false if desc.is_a?(Module) && desc.anonymous?
       name = to_constant_name desc
       return false unless qualified_const_defined? name
       return autoloaded_constants.include?(name)
@@ -492,81 +589,32 @@ module ActiveSupport #:nodoc:
     def new_constants_in(*descs)
       log_call(*descs)
 
-      # Build the watch frames. Each frame is a tuple of
-      #   [module_name_as_string, constants_defined_elsewhere]
-      watch_frames = descs.collect do |desc|
-        if desc.is_a? Module
-          mod_name = desc.name
-          initial_constants = desc.local_constant_names
-        elsif desc.is_a?(String) || desc.is_a?(Symbol)
-          mod_name = desc.to_s
-
-          # Handle the case where the module has yet to be defined.
-          initial_constants = if qualified_const_defined?(mod_name)
-            mod_name.constantize.local_constant_names
-          else
-            []
-          end
-        else
-          raise Argument, "#{desc.inspect} does not describe a module!"
-        end
-
-        [mod_name, initial_constants]
-      end
-
-      constant_watch_stack.concat watch_frames
-
+      constant_watch_stack.watch_namespaces(descs)
       aborting = true
+
       begin
         yield # Now yield to the code that is to define new constants.
         aborting = false
       ensure
-        # Find the new constants.
-        new_constants = watch_frames.collect do |mod_name, prior_constants|
-          # Module still doesn't exist? Treat it as if it has no constants.
-          next [] unless qualified_const_defined?(mod_name)
-
-          mod = mod_name.constantize
-          next [] unless mod.is_a? Module
-          new_constants = mod.local_constant_names - prior_constants
-
-          # Make sure no other frames takes credit for these constants.
-          constant_watch_stack.each do |frame_name, constants|
-            constants.concat new_constants if frame_name == mod_name
-          end
-
-          new_constants.collect do |suffix|
-            mod_name == "Object" ? suffix : "#{mod_name}::#{suffix}"
-          end
-        end.flatten
+        new_constants = constant_watch_stack.new_constants
 
         log "New constants: #{new_constants * ', '}"
+        return new_constants unless aborting
 
-        if aborting
-          log "Error during loading, removing partially loaded constants "
-          new_constants.each { |name| remove_constant name }
-          new_constants.clear
-        end
+        log "Error during loading, removing partially loaded constants "
+        new_constants.each {|c| remove_constant(c) }.clear
       end
 
-      return new_constants
-    ensure
-      # Remove the stack frames that we added.
-      if defined?(watch_frames) && ! watch_frames.blank?
-        frame_ids = watch_frames.collect { |frame| frame.object_id }
-        constant_watch_stack.delete_if do |watch_frame|
-          frame_ids.include? watch_frame.object_id
-        end
-      end
+      return []
     end
 
     class LoadingModule #:nodoc:
       # Old style environment.rb referenced this method directly.  Please note, it doesn't
       # actually *do* anything any more.
       def self.root(*args)
-        if defined?(RAILS_DEFAULT_LOGGER)
-          RAILS_DEFAULT_LOGGER.warn "Your environment.rb uses the old syntax, it may not continue to work in future releases."
-          RAILS_DEFAULT_LOGGER.warn "For upgrade instructions please see: http://manuals.rubyonrails.com/read/book/19"
+        if defined?(Rails) && Rails.logger
+          Rails.logger.warn "Your environment.rb uses the old syntax, it may not continue to work in future releases."
+          Rails.logger.warn "For upgrade instructions please see: http://manuals.rubyonrails.com/read/book/19"
         end
       end
     end
@@ -574,12 +622,12 @@ module ActiveSupport #:nodoc:
     # Convert the provided const desc to a qualified constant name (as a string).
     # A module, class, symbol, or string may be provided.
     def to_constant_name(desc) #:nodoc:
-      name = case desc
-        when String then desc.starts_with?('::') ? desc[2..-1] : desc
+      case desc
+        when String then desc.sub(/^::/, '')
         when Symbol then desc.to_s
         when Module
-          raise ArgumentError, "Anonymous modules have no name to be referenced by" if desc.name.blank?
-          desc.name
+          desc.name.presence ||
+            raise(ArgumentError, "Anonymous modules have no name to be referenced by")
         else raise TypeError, "Not a valid constant descriptor: #{desc.inspect}"
       end
     end
@@ -587,16 +635,15 @@ module ActiveSupport #:nodoc:
     def remove_constant(const) #:nodoc:
       return false unless qualified_const_defined? const
 
-      const = $1 if /\A::(.*)\Z/ =~ const.to_s
-      names = const.to_s.split('::')
-      if names.size == 1 # It's under Object
-        parent = Object
-      else
-        parent = (names[0..-2] * '::').constantize
-      end
+      # Normalize ::Foo, Foo, Object::Foo, and ::Object::Foo to Object::Foo
+      names = const.to_s.sub(/^::(Object)?/, 'Object::').split("::")
+      to_remove = names.pop
+      parent = Inflector.constantize(names * '::')
 
       log "removing constant #{const}"
-      parent.instance_eval { remove_const names.last }
+      constantize(const).before_remove_const if constantize(const).respond_to?(:before_remove_const)
+      parent.instance_eval { remove_const to_remove }
+
       return true
     end
 
