@@ -1,7 +1,11 @@
+require 'active_support/basic_object'
+require 'active_support/core_ext/array/conversions'
+require 'active_support/core_ext/object/acts_like'
+
 module ActiveSupport
-  # Provides accurate date and time measurements using Date#advance and 
-  # Time#advance, respectively. It mainly supports the methods on Numeric,
-  # such as in this example:
+  # Provides accurate date and time measurements using Date#advance and
+  # Time#advance, respectively. It mainly supports the methods on Numeric.
+  # Example:
   #
   #   1.month.ago       # equivalent to Time.now.advance(:months => -1)
   class Duration < BasicObject
@@ -32,8 +36,9 @@ module ActiveSupport
     end
 
     def is_a?(klass) #:nodoc:
-      klass == Duration || super
+      Duration == klass || value.is_a?(klass)
     end
+    alias :kind_of? :is_a?
 
     # Returns true if <tt>other</tt> is also a Duration instance with the
     # same <tt>value</tt>, or if <tt>other == value</tt>.
@@ -46,7 +51,9 @@ module ActiveSupport
     end
 
     def self.===(other) #:nodoc:
-      other.is_a?(Duration) rescue super
+      other.is_a?(Duration)
+    rescue ::NoMethodError
+      false
     end
 
     # Calculates a new Time or Date that is as far in the future
@@ -65,10 +72,12 @@ module ActiveSupport
 
     def inspect #:nodoc:
       consolidated = parts.inject(::Hash.new(0)) { |h,part| h[part.first] += part.last; h }
-      [:years, :months, :days, :minutes, :seconds].map do |length|
+      parts = [:years, :months, :days, :minutes, :seconds].map do |length|
         n = consolidated[length]
         "#{n} #{n == 1 ? length.to_s.singularize : length.to_s}" if n.nonzero?
-      end.compact.to_sentence
+      end.compact
+      parts = ["0 seconds"] if parts.empty?
+      parts.to_sentence(:locale => :en)
     end
 
     protected
@@ -90,7 +99,7 @@ module ActiveSupport
     private
 
       def method_missing(method, *args, &block) #:nodoc:
-        value.send(method, *args)
+        value.send(method, *args, &block)
       end
   end
 end
